@@ -45,6 +45,8 @@ import jcifs.util.LogStream;
 
 public class Config {
 
+public static int socketCount = 0;
+
     /**
      * The static <code>Properties</code>.
      */
@@ -66,6 +68,8 @@ public class Config {
                 in = new FileInputStream( filename );
             }
             Config.load( in );
+            if (in != null)
+                in.close();
         } catch( IOException ioe ) {
             if( log.level > 0 )
                 ioe.printStackTrace( log );
